pub struct AddedPeerInfo {
pub(crate) address: AddrV2,
pub(crate) port: u16,
pub(crate) v1_fallback: bool,
}Expand description
A simple struct of added peers, used to track the ones we added manually by addnode <ip:port> add command.
Fields
address: AddrV2The address of the peer
port: u16The port of the peer
v1_fallback: boolWhether we should allow V1 fallback for this connection